<?php

namespace EventPortal\Providers;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Email\Models\EventEmail;
use EventPortal\Domain\Email\Observers\EventEmailObserver;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Event\Models\EventSpeaker;
use EventPortal\Domain\Util\Menu\Facades\Menu;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Exceptions\Handler;
use EventPortal\Http\Middleware\CanAccessAdmin;
use EventPortal\Http\Middleware\CheckPermission;
use EventPortal\Http\Middleware\CurrentTeam;
use Illuminate\Contracts\Debug\ExceptionHandler;
use Illuminate\Routing\Router;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\ServiceProvider;
use Inertia\Inertia;

class AppServiceProvider extends ServiceProvider
{
    /** @var string */
    protected $root = __DIR__ . '/../..';

    public function boot(Router $router): void
    {
        $router->aliasMiddleware('current-team', CurrentTeam::class);
        $router->aliasMiddleware('check-permission', CheckPermission::class);
        $router->aliasMiddleware('can-access-admin', CanAccessAdmin::class);

        $this->app->booted(function (): void {
            $this->loadRoutesFrom("{$this->root}/routes/routes.php");
        });

        $this->loadViewsFrom("{$this->root}/resources/views", 'EventPortal');

        $this->loadMigrationsFrom("{$this->root}/database/migrations");

        $this->loadFactoriesFrom("{$this->root}/database/factories");

        $this->app->bind(ExceptionHandler::class, Handler::class);

        $this->publishes([
            "{$this->root}/tables/delegate.php" => '/tables/delegates.php',
            "{$this->root}/tables/venue.php" => '/tables/venue.php',
            "{$this->root}/tables/event.php" => '/tables/event.php',
            "{$this->root}/tables/event-delegate.php" => '/tables/event-delegate.php',
        ], 'event-portal-tables');

        $this->publishes([
            "{$this->root}/forms/delegate.php" => '/forms/delegates.php',
            "{$this->root}/forms/event.php" => '/forms/event.php',
            "{$this->root}/forms/venue.php" => '/forms/venue.php',
            "{$this->root}/forms/event-delegate.php" => '/forms/event-delegate.php',
        ], 'event-portal-forms');

        $this->publishes([
            "{$this->root}/sidebars/event.php" => '/sidebars/event.php',
        ], 'event-portal-sidebars');

        $this->publishes([
            "{$this->root}/statistics/events.php" => '/statistics/events.php',
        ], 'event-portal-statistics');

        $this->publishes([
            "{$this->root}/notifications/admin.php" => '/notifications/admin.php',
            "{$this->root}/resources/views/email" => resource_path('views/vendor/eventportal/email'),
        ], 'event-portal-notifications');

        $this->publishes([
            "{$this->root}/config/permission.php" => config_path('/permission.php'),
            "{$this->root}/config/errors.php" => config_path('/errors.php'),
            "{$this->root}/config/state-machine.php" => config_path('/state-machine.php'),
            "{$this->root}/config/audit.php" => config_path('/audit.php'),
            "{$this->root}/config/page-headers.php" => config_path('/page-headers.php'),
            "{$this->root}/config/emails.php" => config_path('/emails.php'),
            "{$this->root}/config/roles.php" => config_path('/roles.php'),
            "{$this->root}/config/permissions.php" => config_path('/permissions.php'),
        ], 'event-portal-config');

        $this->app->bind(ExceptionHandler::class, Handler::class);

        Inertia::setRootView('EventPortal::app');

        $this->setSharedData();

        Route::model('event', Event::class);
        Route::model('delegate', Delegate::class);
        Route::model('venue', Venue::class);
        Route::model('eventDelegate', EventDelegate::class);
        Route::model('admin', Admin::class);
        Route::model('eventSpeaker', EventSpeaker::class);

        EventEmail::observe(EventEmailObserver::class);
    }

    private function setSharedData(): void
    {
        Inertia::share([
            'navigation' => function () {
                return $this->getNavigationData();
            },
            'errors' => function () {
                return session('errors') ? session('errors')->getBag('default')->getMessages() : (object) [];
            },
            'flash' => function () {
                return ['toast' => session('toast')];
            },
        ]);
    }

    private function viewingAdmin(): bool
    {
        return Auth::check() && Auth::user()->isAdmin() && request()->is(['admin', 'admin/*']);
    }

    private function isADelegate(): bool
    {
        return Auth::check() && Auth::user()->isDelegate();
    }

    private function viewingDelegateDashboard(): bool
    {
        return request()->is(['/']);
    }

    /**
     * Returns the data needed for the sites navigation.
     */
    private function getNavigationData(): ?array
    {
        if ($this->viewingAdmin()) {
            return Sidebar::make('primary')->user(Auth::user())->render()['children'];
        }

        if ($this->isADelegate() && $this->viewingDelegateDashboard() === false) {
            return Menu::make('primary')->user(Auth::user())->render()['children'];
        }

        return null;
    }
}
